@@ -22,6 +22,7 @@ const Geocode = imports.gi.GeocodeGlib;
 const GObject = imports.gi.GObject;
 const Soup = imports.gi.Soup;
 
+const OSMNames = imports.osmNames;
 const PhotonParser = imports.photonParser;
 const Place = imports.place;
 const Utils = imports.utils;
@@ -159,6 +160,8 @@ var Overpass = GObject.registerClass({
             properties.postcode = tags['addr:postcode'];
         if (tags['addr:city'])
             properties.city = tags['addr:city'];
+        if (tags['addr:country'])
+            properties.countrycode = tags['addr:country'];
 
         if (tags.place)
             this._setOsmKeyAndValue(properties, tags, 'place');
@@ -200,8 +203,12 @@ var Overpass = GObject.registerClass({
         if (!(element && element.tags))
             return;
 
+        let name = this._getLocalizedName(element.tags, place);
+        if (name)
+            place.name = name;
+
         if (element.tags.name)
-            place.name = element.tags.name;
+            place.nativeName = element.tags.name;
         if (element.tags.population)
             place.population = element.tags.population;
         if (element.tags['contact:website'])
@@ -230,6 +237,13 @@ var Overpass = GObject.registerClass({
             place.note = element.tags.note;
     }
 
+    _getLocalizedName(tags, place) {
+        let language = Utils.getLanguage();
+
+        return OSMNames.getNameForLanguageAndCountry(tags, language,
+                                                     place.country_code);
+    }
+
     _getQueryUrl(osmType, osmId) {
         return Format.vprintf('%s?data=%s', [BASE_URL,
                                              this._generateOverpassQuery(osmType,
