@@ -27,7 +27,7 @@ const GObject = imports.gi.GObject;
 
 const Location = imports.location;
 const Overpass = imports.overpass;
-const Translations = imports.translations;
+const URLS = imports.urls;
 const Utils = imports.utils;
 
 // Matches coordinates string in 'Decimal Degrees' format
@@ -42,14 +42,6 @@ const DMS_COORDINATES_REGEX = new RegExp(
     "i"
 );
 
-// Matches a URL pointing to an object in OpenStreetMap
-const OSM_OBJECT_URL_REGEX =
-    new RegExp(/https?:\/\/(www\.)?openstreetmap\.org\/(node|way|relation)\/(\d+)\/?$/);
-
-// Matches a URL with a specifies coordinate in OpenStreetMap
-const OSM_COORD_URL_REGEX =
-    new RegExp(/https?:\/\/(www\.)?openstreetmap\.org\/?\?(\&?mlat=(\d+(?:\.\d+)?))?(\&mlon=(\d+(?:\.\d+)?))?(\&zoom=(\d+))?(\#map=(\d+)\/(\d+(?:\.\d+)?)\/(\d+(?:\.\d+)?))?/);
-
 var Place = GObject.registerClass(
 class Place extends Geocode.Place {
 
@@ -105,6 +97,9 @@ class Place extends Geocode.Place {
         this._wheelchair = params.wheelchair;
         delete params.wheelchair;
 
+        this._nativeName = params.nativeName;
+        delete params.nativeName;
+
         if (params.place) {
             params = { osm_id: params.place.osm_id,
                        osm_type: params.place.osm_type,
@@ -121,7 +116,7 @@ class Place extends Geocode.Place {
                        state: params.place.state,
                        county: params.place.county,
                        country: params.place.country,
-                       country_code: params.place.contry_code,
+                       country_code: params.place.country_code,
                        continent: params.place.continent };
         }
 
@@ -192,18 +187,10 @@ class Place extends Geocode.Place {
         this._internetAccess = v;
     }
 
-    get openingHoursTranslated() {
-        return Translations.translateOpeningHours(this._openingHours);
-    }
-
     get internetAccess() {
         return this._internetAccess;
     }
 
-    get internetAccessTranslated() {
-        return Translations.translateInternetAccess(this._internetAccess);
-    }
-
     set religion(v) {
         this._religion = v;
     }
@@ -212,10 +199,6 @@ class Place extends Geocode.Place {
         return this._religion;
     }
 
-    get religionTranslated() {
-        return Translations.translateReligion(this._religion);
-    }
-
     set toilets(v) {
         this._toilets = v;
     }
@@ -224,10 +207,6 @@ class Place extends Geocode.Place {
         return this._toilets;
     }
 
-    get toiletsTranslated() {
-        return Translations.translateYesNo(this._toilets);
-    }
-
     set note(v) {
         this._note = v;
     }
@@ -244,10 +223,6 @@ class Place extends Geocode.Place {
         return this._wheelchair;
     }
 
-    get wheelchairTranslated() {
-        return this._translateWheelchair(this._wheelchair);
-    }
-
     get initialZoom() {
         return this._initialZoom;
     }
@@ -260,37 +235,13 @@ class Place extends Geocode.Place {
         this._prefilled = prefilled;
     }
 
-    _translateWheelchair(string) {
-        switch(string) {
-            /* Translators:
-             * This means wheelchairs have full unrestricted access.
-             */
-            case 'yes': return _("yes");
-
-            /* Translators:
-             * This means wheelchairs have partial access (e.g some areas
-             * can be accessed and others not, areas requiring assistance
-             * by someone pushing up a steep gradient).
-             */
-            case 'limited': return _("limited");
-
-            /* Translators:
-             * This means wheelchairs have no unrestricted access
-             * (e.g. stair only access).
-             */
-            case 'no': return _("no");
-
-            /* Translators:
-             * This means that the way or area is designated or purpose built
-             * for wheelchairs (e.g. elevators designed for wheelchair access
-             * only). This is rarely used.
-             */
-            case 'designated': return _("designated");
-
-            default: return null;
-        }
+    get nativeName() {
+        return this._nativeName;
     }
 
+    set nativeName(nativeName) {
+        this._nativeName = nativeName;
+    }
 
     toJSON() {
         let bounding_box = null;
@@ -311,6 +262,7 @@ class Place extends Geocode.Place {
                  osm_type: this.osm_type,
                  place_type: this.place_type,
                  name: this.name,
+                 nativeName: this.nativeName,
                  bounding_box: bounding_box,
                  this_type: this.this_type,
                  location: location,
@@ -323,7 +275,7 @@ class Place extends Geocode.Place {
                  state: this.state,
                  county: this.county,
                  country: this.country,
-                 country_code: this.contry_code,
+                 country_code: this.country_code,
                  continent: this.continent,
                  population: this.population,
                  website: this.website,
@@ -454,71 +406,44 @@ let overpass = null;
  */
 const Application = imports.application;
 
-function _parseOSMObjectURL(match, callback) {
-    let [,, type, id] = match;
-    let storedPlace = Application.placeStore.existsWithOsmTypeAndId(type, id);
-
-    if (storedPlace) {
-        callback(storedPlace, null);
-        return;
-    }
-
-    if (overpass === null)
-        overpass = new Overpass.Overpass();
-
-    Application.application.mark_busy();
-    overpass.fetchPlace(type, id, (place) => {
-        Application.application.unmark_busy();
-        if (place)
-            callback(place, null);
-        else
-            callback(null, _("Place not found in OpenStreetMap"));
-    });
-}
-
-function _parseOSMCoordURL(match, callback) {
-    let [,,,mlat,,mlon,,zoom,,mapZoom,mapLat, mapLon] = match;
-    let lat;
-    let lon;
-    let z;
-
-    if (mapZoom && mapLat && mapLon) {
-        lat = mapLat;
-        lon = mapLon;
-        z = mapZoom;
-    } else if (mlat && mlon) {
-        lat = mlat;
-        lon = mlon;
-        if (zoom)
-            z = zoom;
-    } else {
-        callback(null, _("OpenStreetMap URL is not valid"));
-        return;
-    }
-
-    if (!Place.validateCoordinates(lat, lon)) {
-        callback(null, _("Coordinates in URL are not valid"));
-        return;
-    }
-
-    let location = new Location.Location({ latitude: lat, longitude: lon });
-    let place = z ? new Place({ location: location, initialZoom: z }) :
-                    new Place({ location: location });
-
-    callback(place, null);
-}
-
 function parseHttpURL(text, callback) {
-    let match = text.match(OSM_OBJECT_URL_REGEX);
+    let [type, id] = URLS.parseAsObjectURL(text);
 
-    if (match) {
-        _parseOSMObjectURL(match, callback);
+    if (type && id) {
+        let storedPlace = Application.placeStore.existsWithOsmTypeAndId(type, id);
+
+        if (storedPlace) {
+            callback(storedPlace, null);
+            return;
+        }
+
+        if (overpass === null)
+            overpass = new Overpass.Overpass();
+
+        Application.application.mark_busy();
+        overpass.fetchPlace(type, id, (place) => {
+            Application.application.unmark_busy();
+            if (place)
+                callback(place, null);
+            else
+                callback(null, _("Place not found in OpenStreetMap"));
+        });
     } else {
-        match = text.match(OSM_COORD_URL_REGEX);
-        if (match)
-            _parseOSMCoordURL(match, callback);
-        else
+        let [lat, lon, zoom] = URLS.parseAsCoordinateURL(text);
+
+        if (lat && lon) {
+            if (!Place.validateCoordinates(lat, lon)) {
+                callback(null, _("Coordinates in URL are not valid"));
+            } else {
+                let location = new Location.Location({ latitude: lat, longitude: lon });
+                let place = zoom ? new Place({ location: location, initialZoom: zoom }) :
+                                   new Place({ location: location });
+
+                callback(place, null);
+            }
+        } else {
             callback(null, _("URL is not supported"));
+        }
     }
 }
 
